﻿Option Explicit On
Option Strict On

Imports Microsoft.VisualBasic

''' <summary>
''' Provides access to common user-related information.
''' All methods/properties in this class are declared as "Shared".
''' </summary>
''' <remarks></remarks>
Public Class cUserProps

#Region " --- Constants --- "

   ''' <summary>
   ''' This commandline argument may be used to run the application as a user whose name is not being
   ''' taken from the Windows login.
   ''' To use, setup a shortcut that contains the following constant, followed by the user-name as it 
   ''' is to be used!
   ''' </summary>
   ''' <remarks></remarks>
   Private Const _conCommandlineArg_UserLogin As String = "/SetUser="

#End Region

#Region " --- Variables --- "

   ''' <summary>
   ''' Will contain the logon name that is to be used for:
   ''' - storing the Create-/Last Update-/Delete-user
   ''' - checking as to whether the user has any additional permissions (user-table / SQLS only)
   ''' This variable is only being used in the Windows_LogonName-property.
   ''' </summary>
   ''' <remarks></remarks>
   Private Shared _strUserName As String = String.Empty

   ''' <summary>
   ''' Will contain all existing membership-groups for the current user.
   ''' The list will be filled upon calling "User_IsMemberOf" for the first time (only if connecting to SQLS).
   ''' </summary>
   ''' <remarks></remarks>
   Private Shared _dtUser_GroupMemberships As DataTable

#End Region

#Region " --- Methods and Properties --- "

   ''' <summary>
   ''' Returns the Windows Logon Name (without the preceeding Domain-/Workgroup name)
   ''' </summary>
   Private Shared ReadOnly Property GetUserName() As String
      Get
         If TypeOf My.User.CurrentPrincipal Is Security.Principal.WindowsPrincipal Then
            'The application is using Windows authentication.
            'The name format is DOMAIN\USERNAME.
            Dim arrstrParts() As String = Split(My.User.Name, "\")
            '-> Dim strDomainName As String = arrstrParts(0)
            Dim strUserName As String = arrstrParts(1)
            Return strUserName
         Else
            'The application is using custom authentication.
            Return My.User.Name
         End If
      End Get
   End Property

   ''' <summary>
   ''' Returns the current user's logon name. This is usually the Windows logon name, but can be overrided
   ''' by the "SetUser"-commandline argument (see the definition of the _conCommandlineArg_UserLogin constant).
   ''' </summary>
   Public Shared ReadOnly Property Windows_LogonName(Optional ByVal fAppendComputerName As Boolean = False) As String
      Get
         If _strUserName = String.Empty Then
            With My.Application.CommandLineArgs
               For Each strCommandLineArg As String In My.Application.CommandLineArgs
                  If strCommandLineArg.StartsWith(_conCommandlineArg_UserLogin) Then
                     _strUserName = strCommandLineArg.Remove(0, _conCommandlineArg_UserLogin.Length)
                     If _strUserName.StartsWith("=") Then _strUserName = _strUserName.Remove(0, 1)
                     MsgBox("You are acting as user """ & _strUserName & """.", MsgBoxStyle.Information, Application.ProductName & ": User")
                     Exit For
                  End If
               Next
               'If the appropriate command-line arg has not been found, use the windows user-name.
               If _strUserName = String.Empty Then _strUserName = GetUserName
            End With
         End If

         If fAppendComputerName Then
            Return _strUserName & " (" & My.Computer.Name & ")"
         Else
            Return _strUserName
         End If

      End Get
   End Property

   '*** Additional user privileges: Commented out for this sample! ***
   '''' <summary>
   '''' Returns whether or not the given user is a member of the specified permissions group. 
   '''' </summary>
   '''' <param name="PermissionGroup">The group to be tested.</param>
   '''' <param name="DoRefresh">If this flag is True, the function will requery the local DataTable, i.e. after a change in the connectivity.</param>
   '''' <returns>True: User is a member of the specified group.</returns>
   '''' <remarks></remarks>
   'Public Shared Function Permissions_IsMemberOf( _
   '            ByVal PermissionGroup As cSQLS_Permissions_Administration.ePermissionGroups, _
   '            Optional ByVal DoRefresh As Boolean = False, _
   '            Optional ByVal fShowMessageIfPermissionsInsufficient As Boolean = False) As Boolean
   '   If Not My.Settings.UseSQLS Then
   '      'While using JET, there is no group-memberships, so always return False.
   '      Return False
   '   Else
   '      If _dtUser_GroupMemberships Is Nothing OrElse DoRefresh Then
   '         'First call - initialize the local DataTable (read all groups from the database in which the user is listed as a member).
   '         Dim cPA As New cSQLS_Permissions_Administration
   '         _dtUser_GroupMemberships = New DataTable
   '         _dtUser_GroupMemberships = cPA.GroupsAndMemberships_DataTable(Windows_LogonName)
   '      End If

   '      If _dtUser_GroupMemberships Is Nothing Then
   '         'Shouldn't ever happen (the DataTable may be empty, but it should be there).
   '         If fShowMessageIfPermissionsInsufficient Then PermissionsInsufficient_ShowMessage()
   '         Return False
   '      Else
   '         'Select all rows (will always be a single row) that correspond with the requested right
   '         Dim arrRows As DataRow() = _dtUser_GroupMemberships.Select("PG_ID = " & CType(PermissionGroup, Integer).ToString)

   '         If arrRows.Length > 0 Then
   '            'Return the flag as found in the datatable
   '            '1. Check whether user is an explicit member
   '            Dim fIsMember As Boolean = _
   '               CType(arrRows(0)(cSQLS_Permissions_Administration.FieldName(cSQLS_Permissions_Administration.eFields_GroupMemberships_DataTable.IsMember)), Boolean)
   '            If Not fIsMember AndAlso PermissionGroup <> cSQLS_Permissions_Administration.ePermissionGroups.Admins Then
   '               'Check whether user is an implicit member (i.e. a member of the Admins group)
   '               arrRows = _dtUser_GroupMemberships.Select("PG_ID = " & CType(cSQLS_Permissions_Administration.ePermissionGroups.Admins, Integer).ToString)
   '               fIsMember = CType(arrRows(0)(cSQLS_Permissions_Administration.FieldName(cSQLS_Permissions_Administration.eFields_GroupMemberships_DataTable.IsMember)), Boolean)
   '            End If
   '            Return fIsMember
   '         Else
   '            'Group not found (shouldn't happen - all groups should be part of the DataTable)
   '            If fShowMessageIfPermissionsInsufficient Then PermissionsInsufficient_ShowMessage()
   '            Return False
   '         End If
   '      End If
   '   End If
   'End Function

   'Private Shared Sub PermissionsInsufficient_ShowMessage()
   '   MsgBox("You do not have sufficient permissions to use this form." & vbCrLf & vbCrLf & _
   '      "Action canceled.", MsgBoxStyle.Exclamation, Application.ProductName & ": Access denied.")
   'End Sub

   Public Shared Function Permissions_DeletionAllowed(ByVal strCreateUser As String) As Boolean
      If Windows_LogonName = strCreateUser Then
         'The user attempting to delete is the same one that originally created the record - this is OK.
         Return True
      Else
         '*** Additional user privileges: Commented out for this sample! ***
         ''The user attempting to delete is not the one that originally created the record. 
         ''Check the permissions - membership in the "Extended" group is required
         'If Permissions_IsMemberOf(cSQLS_Permissions_Administration.ePermissionGroups.Extended) Then
         '   'The current user is a member - allow deleting.
         '   Return True
         'Else
         '   MsgBox("You do not have sufficient permissions to delete this record." & vbCrLf & _
         '      "In order to delete, you must either be the person that originally created" & vbCrLf & _
         '      "the record or you must have extended privileges." & vbCrLf & vbCrLf & _
         '      "Action canceled.", MsgBoxStyle.Exclamation, Application.ProductName & ": Permission denied")
         '   Return False
         'End If
         Return True
      End If
   End Function

   Public Shared Function Permissions_EditAllowed(ByVal strCreateUser As String) As Boolean
      If Windows_LogonName = strCreateUser Then
         'The user attempting to edit is the same one that originally created the record - this is OK.
         Return True
      Else
         '*** Additional user privileges: Commented out for this sample! ***
         'The user attempting to edit is not the one that originally created the record. 
         'Check the permissions - membership in the "Extended" group is required
         'If Permissions_IsMemberOf(cSQLS_Permissions_Administration.ePermissionGroups.Extended) Then
         '   'The current user is a member - allow deleting.
         '   Return True
         'Else
         MsgBox("You do not have sufficient permissions to edit this record." & vbCrLf & _
            "In order to be able to change this record, you must either be the person that holds" & vbCrLf & _
            "ownership of the record or you must have extended privileges." & vbCrLf & vbCrLf & _
            "Action canceled.", MsgBoxStyle.Exclamation, Application.ProductName & ": Permission denied")
         Return False
         'End If
      End If

   End Function

#End Region

End Class
