﻿Option Explicit On
Option Strict On

Imports System
Imports System.Data
Imports System.Data.SqlClient

''' <summary>
''' Beispiel für eine Master-Detail Beziehung über zwei Gridviews.
''' Zwecks eines übersichtlichen Beispiels findet *AUSSCHLIESSLICH* lesender
''' Zugriff statt - Änderungen werden also ggf. nicht in die DB übernommen.
''' </summary>
Public Class frm01_Categories

   'Lokale Tabelle für die Combobox im Detail-Grid (muss nur einmalig geladen werden)
   Private _dtSuppliers As DataTable

   Private Sub frmCategories_Load( _
            ByVal sender As System.Object, ByVal e As System.EventArgs) _
            Handles MyBase.Load
      'Master-Grid (Categories) befüllen
      Using cn As New SqlConnection(My.Settings.SQLS_DB)
         Using cmd As New SqlCommand( _
               "SELECT CategoryID, CategoryName, Description, Picture " & _
               "FROM dbo.Categories " & _
               "ORDER BY CategoryName", _
               cn)
            Using da As New SqlDataAdapter(cmd)
               Dim dt As New DataTable("Categories")
               da.Fill(dt)
               'Die Spalten des Grids sollen sich am Inhalt der Daten orientieren
               '(= Standard, hier nur der Deutlichkeit halber aufgeführt)
               gvMaster_Categories.AutoGenerateColumns = True
               'Nun noch das Grid an die DataTable binden:
               gvMaster_Categories.DataSource = dt
            End Using
         End Using

         With gvMaster_Categories
            'ID-Spalte ausblenden
            .Columns(0).Visible = False
            With .Columns(1)
               .HeaderText = "Category"
               .AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells
            End With
            '"Description" soll den restlichen horizontal zur Verfügung stehenden
            'Platz ausnutzen
            With .Columns(2)
               .HeaderText = "Description"
               .AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill
            End With
            With .Columns(3)
               .HeaderText = "Picture"
               .AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells
            End With

            'Die Zeilenhöhe soll sich nach dem Inhalt richten, aber nur
            'für Datenzeilen, nicht für die Spaltenköpfe.
            .AutoResizeRows(DataGridViewAutoSizeRowsMode.AllCellsExceptHeaders)
         End With

         'Die lokale Tabelle mit den Suppliers befüllen (für Combobox im Detail-Grid)
         Using cmd As New SqlCommand( _
               "SELECT SupplierID, CompanyName " & _
               "FROM dbo.Suppliers " & _
               "ORDER BY CompanyName", _
               cn)
            Using da As New SqlDataAdapter(cmd)
               _dtSuppliers = New DataTable("Suppliers")
               da.Fill(_dtSuppliers)
            End Using
         End Using
      End Using

      'Erste Zeile im oberen Grid (Master) auswählen und Detail-Grid 
      'aktualisieren
      gvMaster_Categories.Rows(0).Selected = True
   End Sub

   ''' <summary>
   ''' Wird angesprungen, wenn eine Zeile im Grid selektiert wird, 
   ''' d.h. der Benutzer irgendein Element in einer anderen als der
   ''' aktuellen Zeile anwählt.
   ''' Aufgabe hier: abhängige Grids aktualisieren
   ''' </summary>
   ''' <remarks>Vorsicht ist geboten: Dieser Event wird IMMER angesprungen,
   ''' selbst wenn (wörtlich nehmen!) "keine" Zeile angesprungen wird!</remarks>
   Private Sub gvMaster_Categories_RowEnter( _
                  ByVal sender As Object, _
                  ByVal e As System.Windows.Forms.DataGridViewCellEventArgs) _
               Handles gvMaster_Categories.RowEnter
      'Falls eben ein neuer DS bearbeitet wurde und der Fokus auf ein 
      'Detail-Control gelegt wurde, wird zwar ein existenter Master-DS
      'angewählt, die Zellenauswahl verbleibt aber wie gehabt. 
      'Daher hier prüfen und die Auswahl ggf. zurücksetzen
      If gvMaster_Categories.CurrentCellAddress.Y <> e.RowIndex Then
         gvMaster_Categories.ClearSelection()
      End If

      'Beispiel für das code-gesteuerte Hinzufügen und Binden von Spalten 
      With gvDetail_Products
         'Alle Spalten (ggf.) löschen
         While .Columns.Count > 0 : .Columns.Remove(.Columns(0)) : End While

         'Das Erzeugen und Binden der Spalten übernehmen wir hier im Code
         .AutoGenerateColumns = False

         'Spalten einfügen und bnden
         Dim colProductID As New DataGridViewTextBoxColumn
         With colProductID
            .Name = "ProductID"
            .DataPropertyName = "ProductID"
            .HeaderText = "Product ID"
            '.Visible = False <- hier könnte man diese Spalte auch ausblenden
            .AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells
         End With
         .Columns.Add(colProductID)

         Dim colProductName As New DataGridViewTextBoxColumn
         With colProductName
            .Name = "ProductName"
            .DataPropertyName = "ProductName"
            .HeaderText = "Product Name"
            .AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill
         End With
         .Columns.Add(colProductName)

         Dim colSupplierID As New DataGridViewComboBoxColumn
         With colSupplierID
            .Name = "SupplierID"
            'Datenquelle ("RowSource") der Combobox
            .DataSource = _dtSuppliers
            'Binden des Feldes an die Spalte im Grid
            .DataPropertyName = "SupplierID"
            'Einstellen der Combobox
            .ValueMember = "SupplierID" '-> Feld, das die Combo an das Grid übergeben soll
            .DisplayMember = "CompanyName" '-> Feld, das der Benutzer sehen soll
            .HeaderText = "Supplier"
            .AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill
            'Nachfolgende Zeile aktivieren, um die Combo nur anzuzeigen, 
            'wenn die Zelle selektiert wurde:
            '.DisplayStyle = DataGridViewComboBoxDisplayStyle.Nothing
         End With
         .Columns.Add(colSupplierID)
      End With

      'Nun die Products-Datensätze (entspr. gewählter Category) laden ...
      Dim intCategoryID As String = String.Empty
      If Not gvMaster_Categories.Rows(e.RowIndex).Cells(0).Value Is Nothing Then
         intCategoryID = _
            gvMaster_Categories.Rows(e.RowIndex).Cells(0).Value.ToString
      End If
      Using cn As New SqlConnection(My.Settings.SQLS_DB)
         Using cmd As New SqlCommand( _
               "SELECT ProductID, ProductName, SupplierID " & _
               "FROM dbo.Products " & _
               "WHERE CategoryID = " & intCategoryID.ToString & _
               "ORDER BY ProductName ", _
               cn)
            Using da As New SqlDataAdapter(cmd)
               Dim dt As New DataTable("Products")
               da.Fill(dt)
               '... und das Grid entspr. binden
               gvDetail_Products.DataSource = dt
            End Using
         End Using
      End Using

      'Die DataSource des GridViews lässt sich ohne Weiteres auch wieder in eine
      'DataTable zurückkonvertieren.
      'Hier mal, um beispielhaft die Anzahl der Zeilen im Detail-Grid anzuzeigen,
      'was man normalerweise natürlich direkt über die DataTable machen würde).
      lblDetail_Info_RowCount.Text = _
         DirectCast(gvDetail_Products.DataSource, DataTable).Rows.Count & _
         " Produkt(e) gefunden."
   End Sub

End Class