Option Strict On
Option Explicit On

Public Class frmPleaseWait

   Public Enum eCancelAction
      None = 0
      HideForm
      CloseForm
   End Enum

   Private _frmCaller As Form = Nothing
   Private _fAllowClose As Boolean = False
   Private _fReEnableCallerWhenClosing As Boolean = True
   Private _fUserCanceled As Boolean = False
   Private _fProcessFinished As Boolean = False
   Private _CancelAction As eCancelAction = eCancelAction.None
   Private _strCanceling_StatusText As String = String.Empty

   Public Event Canceled()

   ''' <summary>
   ''' Returns a reference to the local ProgressBar control
   ''' </summary>
   ''' <value></value>
   ''' <returns></returns>
   ''' <remarks></remarks>
   Public ReadOnly Property ProgressBar() As Windows.Forms.ProgressBar
      Get
         Return pb
      End Get
   End Property

   Public ReadOnly Property UserCanceled() As Boolean
      Get
         Return _fUserCanceled
      End Get
   End Property

   ''' <summary>
   ''' Can be used from i.e. a separate thread to avoid having to use delegates for having the worker-thread signal that the operation has finished.
   ''' </summary>
   ''' <value></value>
   ''' <returns></returns>
   ''' <remarks></remarks>
   Public Property ProcessFinished() As Boolean
      Get
         Return _fProcessFinished
      End Get
      Set(ByVal value As Boolean)
         _fProcessFinished = value
      End Set
   End Property

   ''' <summary>
   ''' Allows for explicitly showing/hiding the progressbar
   ''' </summary>
   ''' <value></value>
   ''' <returns></returns>
   ''' <remarks></remarks>
   Public Property Progressbar_Visible() As Boolean
      Get
         Return pb.Visible
      End Get
      Set(ByVal value As Boolean)
         If value <> pb.Visible Then
            If value Then
               Me.Height += pb.Height
            Else
               Me.Height -= pb.Height
            End If
            Me.Invalidate()
         End If
         pb.Visible = value
      End Set
   End Property

   ''' <summary>
   ''' Instance of the form that "owns" the wait-form.
   ''' This form will be disabled while the wait-form is being shown.
   ''' </summary>
   ''' <value></value>
   ''' <returns></returns>
   ''' <remarks></remarks>
   Public Property CallingForm() As Form
      Get
         Return _frmCaller
      End Get
      Set(ByVal value As Form)
         'If the "old" form is disabled, re-enable it
         If _frmCaller IsNot Nothing AndAlso _frmCaller.Enabled = False Then _frmCaller.Enabled = True

         _frmCaller = value
         If _frmCaller IsNot Nothing Then _frmCaller.Enabled = False
      End Set
   End Property

   ''' <summary>
   ''' Shows a wait-message along with an updatable status-message.
   ''' </summary>
   ''' <param name="strMessage">Anzuzeigende Meldung</param>
   ''' <param name="frmCaller">Aufrufendes Formular</param>
   ''' <param name="fAllowCancel">If True, a Cancel-button will be shown. If clicked, the Canceled-event will be fired.
   ''' Alternatively, the visibility of the form may be checked by callers where this form is not locally declared as "WithEvents".</param>
   ''' <remarks></remarks>
   Public Sub New( _
            ByVal strMessage As String, _
            ByVal frmCaller As Form, _
            Optional ByVal fReEnableCallerWhenClosing As Boolean = True, _
            Optional ByVal fAllowCancel As Boolean = False, _
            Optional ByVal CancelAction As eCancelAction = eCancelAction.None, _
            Optional ByVal strCanceling_StatusText As String = "Canceling, please stand by ...", _
            Optional ByVal fShowProgressbar As Boolean = False)
      Me.InitializeComponent()

      Me.SetMessage(strMessage)
      With Me
         '.TopMost = True

         cmdCancelProcess.Visible = fAllowCancel
         cmdCancelProcess.Enabled = fAllowCancel
         _strCanceling_StatusText = strCanceling_StatusText
         Progressbar_Visible = fShowProgressbar

         If Not frmCaller Is Nothing Then
            CallingForm = frmCaller
            Me.Icon = CallingForm.Icon

            'CallingForm.Owner = Me

            'Application.UseWaitCursor = True
            'Cursor.Current = Cursors.WaitCursor

            Me.Refresh()
            CallingForm.Refresh()

            'Application.DoEvents()
            CallingForm.Enabled = False
            _fReEnableCallerWhenClosing = fReEnableCallerWhenClosing
         End If

         .Visible = True
         Cursor.Current = Cursors.WaitCursor
         Me.Left = CallingForm.Left + CType(((CallingForm.Width - Me.Width) / 2), Integer)
         Me.Top = CallingForm.Top + CType(((CallingForm.Height - Me.Height) / 2), Integer)
         .SetMessage(strMessage)
      End With
   End Sub

   ''' <summary>
   ''' Updates the status-message.
   ''' </summary>
   ''' <param name="strMessage">Anzuzeigende Meldung</param>
   ''' <remarks></remarks>
   Public Sub SetMessage(ByVal strMessage As String) ', ByRef frmCaller As Form)
      lblStatus.Text = strMessage
      Me.Refresh()
   End Sub

   ''' <summary>
   ''' Closes the wait-form.
   ''' </summary>
   ''' <remarks>MUST be called in order to re-activate the calling form!
   ''' Alternatively, the .Dispose-method can do all clean-up required.</remarks>
   Public Sub CloseForm()
      Me.Reset()
   End Sub

   Private Sub Reset()
      If Not CallingForm Is Nothing Then
         If _fReEnableCallerWhenClosing Then CallingForm.Enabled = True
         CallingForm.Focus()
         CallingForm.Owner = Nothing
      End If
      _fAllowClose = True
      Application.UseWaitCursor = False
      Me.Close()
      CallingForm = Nothing
   End Sub

   Private Sub frmPleaseWait_Disposed(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Disposed
      Reset()
   End Sub

   Private Sub frmPleaseWait_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
      e.Cancel = (Not _fAllowClose)
   End Sub

   Private Sub frmPleaseWait_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint
      Dim pDividerLines As New Pen(Color.Navy, 1)
      'Dim pOuterFrame As New Pen(Color.LightGray, 2)
      Dim pOuterFrame As New Pen(Color.Navy, 1)
      Dim rec As New Rectangle
      Const conMargin As Single = 5

      With e.Graphics.ClipBounds
         'Divider line between header and status-text
         e.Graphics.DrawLine(pDividerLines, conMargin * 2, lblStatus.Top - conMargin, .Width - conMargin * 2, lblStatus.Top - conMargin)

         'Frame around the form
         rec = New Rectangle(CType(.Left + 1, Integer), CType(.Top + 1, Integer), CType(.Right - 3, Integer), CType(.Bottom - 3, Integer))
         e.Graphics.DrawRectangle(pOuterFrame, rec)
      End With
      pDividerLines.Dispose()
      pOuterFrame.Dispose()
   End Sub

   Private Sub cmdCancelProcess_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdCancelProcess.Click
      cmdCancelProcess.Enabled = False
      lblStatus.Text = _strCanceling_StatusText
      _fUserCanceled = True
      RaiseEvent Canceled()

      Select Case _CancelAction
         Case eCancelAction.CloseForm
            Me.Reset()
         Case eCancelAction.HideForm
            Me.Visible = False
         Case Else '-> eCancelAction.None
            'Do nothing
      End Select
   End Sub

End Class